using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.IO;
using System.Text.RegularExpressions;

namespace thumbsGenerator
{
	/// <summary>
	/// Pagina aspx per ridimensionare al volo le immagini
    /// <param name="FN">Path (FileName) dell'immagine da ridimensionare</param>
    /// <param name="MW">Larghezza (Width) dell'immagine che si vuole ottenere</param>
    /// <param name="MH">Altezza (Height) dell'immagine che si vuole ottenere</param>
    /// </summary>
	public partial class thumbnail : System.Web.UI.Page
	{
		int lWidth = 0; // Larghezza massima dopo il quale l'immagine viene ridimensionata
		int lHeight = 0; // Altezza massima dopo il quale l'immagine viene ridimensionata

		double xfactor;
		double yfactor;
		double factor;

		//Oggetti Images per la modifica
		System.Drawing.Image oldImage, newImage;

        private Image getThumbnailImage(Image src, Int32 width, Int32 height){
            return src.GetThumbnailImage(width, height, null, IntPtr.Zero);	//IntPtr.Zero
        }

		private void Page_Load(object sender, System.EventArgs e) 
		{
            if(Request.QueryString["FN"] != null) {
                String originalFileName = Request.QueryString["FN"].ToLower();
                String decodedFileName = Server.UrlDecode(originalFileName);


                //Leggo i parametri i li converto...
                if(Request.QueryString["MW"] != null && Request.QueryString["MW"].ToString() != "" && IsInteger(Convert.ToString(Request.QueryString["MW"]))) lWidth = Convert.ToInt32(Request.QueryString["MW"]);
                if(Request.QueryString["MH"] != null && Request.QueryString["MH"].ToString() != "" && IsInteger(Convert.ToString(Request.QueryString["MH"]))) lHeight = Convert.ToInt32(Request.QueryString["MH"]);

                if(decodedFileName != "") {

                    try {
                        oldImage = System.Drawing.Image.FromFile(Server.MapPath(decodedFileName), true);

                        //Controllo i parametri a 0
                        if(lWidth == 0) lWidth = oldImage.Width;
                        if(lHeight == 0) lHeight = oldImage.Height;

                        //Controllo se l'iimagine deve essere ridimensionata...
                        if(oldImage.Width > lWidth || oldImage.Height > lHeight) {
                            xfactor = (1.0 * oldImage.Width / lWidth);
                            yfactor = (1.0 * oldImage.Height / lHeight);

                            if(xfactor >= yfactor) {
                                factor = xfactor;
                            } else {
                                factor = yfactor;
                            }

                            //Adesso calcolo le dimensioni in base al fattore...
                            lWidth = Convert.ToInt32(oldImage.Width / factor);
                            lHeight = Convert.ToInt32(oldImage.Height / factor);

                            // Se richiedo una Thumbnail troppo grande riduco l'immagine manualmente
                            if(lWidth <= 120 && lHeight <= 120) {

                                // Immagine piccola, visualizzo la miniatura...
                                // Bug di GetThumbnailImage, se ritorno un OutOfMemory,  necessario rieseguirla.
                                try {
                                    newImage = getThumbnailImage(oldImage, lWidth, lHeight);
                                } catch (OutOfMemoryException){
                                    newImage = getThumbnailImage(oldImage, lWidth, lHeight);
                                }

                            } else {
                                //Creo la bitmap nuova che conterr l'immagine
                                Bitmap bmp = new Bitmap(lWidth, lHeight);
                                bmp.SetResolution(oldImage.HorizontalResolution, oldImage.VerticalResolution);

                                //Mi serve l'oggetto Graphics					
                                Graphics grFx = Graphics.FromImage(bmp);

                                //Impostazioni globali...
                                grFx.CompositingQuality = CompositingQuality.HighQuality;
                                grFx.CompositingMode = CompositingMode.SourceOver;
                                grFx.InterpolationMode = InterpolationMode.HighQualityBicubic;
                                grFx.PageUnit = GraphicsUnit.Pixel;
                                grFx.SmoothingMode = SmoothingMode.HighQuality;

                                //Ridimensionamento
                                grFx.DrawImage(oldImage, 0, 0, lWidth, lHeight);
                                grFx.Dispose();

                                newImage = bmp;
                            }
                        } else
                            newImage = oldImage;

                        // Controllo se si vuole disegnare il logo sopra le immagini ridimensionate
                        Boolean useLogoOver = (Boolean)Application["useLogoOverImages"];
                        Graphics grxImage = Graphics.FromImage(newImage);

                        if(useLogoOver) {

                            // Logo con sfondo trasparente (nel mio caso ho usato una png)
                            string logoPath = Application["LogoOverImagesPath"].ToString();
                            Image logo = Image.FromFile(Server.MapPath(logoPath), true);

                            // 1.0 * nn  un modo veloce per convertire un intero in float...
                            float sizex = (float)(1.0 * newImage.Width / 4);
                            float sizeh = sizex * (float)(1.0 * logo.Height / logo.Width);

                            // Idem, il 3.8 deve essere modificato a seconda delle esigenze
                            // nel mio caso il logo viene disegnato in basso a destra
                            float a = (float)newImage.Width - (float)(newImage.Width / 3.8);
                            float b = (float)(newImage.Height - sizeh - 5);

                            PointF uPoint = new PointF(a, b);
                            SizeF sizelogo = new SizeF(sizex, sizeh);

                            // Disegno il logo sopra la nuova immagine
                            grxImage.DrawImage(logo, new RectangleF(uPoint, sizelogo));
                        }



                        //Seleziono il content type corretto ed invio l'immagine sullo stream
                        switch(Path.GetExtension(Server.MapPath(decodedFileName))) {
                            case ".jpg":
                                Response.ContentType = "image/jpeg";	//jpeg
                                newImage.Save(Response.OutputStream, ImageFormat.Jpeg);	//Jpeg
                                break;
                            case ".jpeg":
                                Response.ContentType = "image/jpeg";	//jpeg
                                newImage.Save(Response.OutputStream, ImageFormat.Jpeg);	//Jpeg
                                break;
                            case ".gif":
                                Response.ContentType = "image/gif";		//gif
                                newImage.Save(Response.OutputStream, ImageFormat.Gif);	//Gif
                                break;
                            case ".png":
                                Response.ContentType = "image/png";		//Png
                                newImage.Save(Response.OutputStream, ImageFormat.Png);	//Png
                                break;
                            case ".bmp":
                                Response.ContentType = "image/bmp";		//Bmp
                                newImage.Save(Response.OutputStream, ImageFormat.Bmp);	//Bmp
                                break;
                            default:
                                Response.ContentType = "image/*";	//else
                                newImage.Save(Response.OutputStream, ImageFormat.MemoryBmp);
                                break;
                        }

                        //Distruzione oggetti
                        oldImage.Dispose();
                        newImage.Dispose();

                        oldImage = null;
                        newImage = null;

                    } catch {
                        // In questo punto, se si vuole si pu passare un'immagine di errore...
                        // oppure, non fare nulla.
                    }
                }
            }
		}


		private bool IsInteger(string strTmp) 
		{
			Regex objNotIntPattern = new Regex("[^0-9-]");
			Regex objIntPattern = new Regex("^-[0-9]+$|^[0-9]+$");
			return !objNotIntPattern.IsMatch(strTmp) && objIntPattern.IsMatch(strTmp);
		}

		#region Codice generato da Progettazione Web Form
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: questa chiamata  richiesta da Progettazione Web Form ASP.NET.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Metodo necessario per il supporto della finestra di progettazione. Non modificare
		/// il contenuto del metodo con l'editor di codice.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
